@@ -41,10 +41,10 @@ bool SkeletonProfile::_set(const StringName &p_path, const Variant &p_value) {
 
 		if (what == "group_name") {
 			set_group_name(which, p_value);
+			return true;
 		} else if (what == "texture") {
 			set_texture(which, p_value);
-		} else {
-			return false;
+			return true;
 		}
 	}
 
@@ -55,25 +55,31 @@ bool SkeletonProfile::_set(const StringName &p_path, const Variant &p_value) {
 
 		if (what == "bone_name") {
 			set_bone_name(which, p_value);
+			return true;
 		} else if (what == "bone_parent") {
 			set_bone_parent(which, p_value);
+			return true;
 		} else if (what == "tail_direction") {
 			set_tail_direction(which, static_cast<TailDirection>((int)p_value));
+			return true;
 		} else if (what == "bone_tail") {
 			set_bone_tail(which, p_value);
+			return true;
 		} else if (what == "reference_pose") {
 			set_reference_pose(which, p_value);
+			return true;
 		} else if (what == "handle_offset") {
 			set_handle_offset(which, p_value);
+			return true;
 		} else if (what == "group") {
 			set_group(which, p_value);
+			return true;
 		} else if (what == "require") {
 			set_required(which, p_value);
-		} else {
-			return false;
+			return true;
 		}
 	}
-	return true;
+	return false;
 }
 
 bool SkeletonProfile::_get(const StringName &p_path, Variant &r_ret) const {
@@ -86,42 +92,50 @@ bool SkeletonProfile::_get(const StringName &p_path, Variant &r_ret) const {
 
 		if (what == "group_name") {
 			r_ret = get_group_name(which);
+			return true;
 		} else if (what == "texture") {
 			r_ret = get_texture(which);
-		} else {
-			return false;
+			return true;
 		}
-	}
-
-	if (path.begins_with("bones/")) {
+	} else if (path.begins_with("bones/")) {
 		int which = path.get_slicec('/', 1).to_int();
 		String what = path.get_slicec('/', 2);
 		ERR_FAIL_INDEX_V(which, bones.size(), false);
 
 		if (what == "bone_name") {
 			r_ret = get_bone_name(which);
+			return true;
 		} else if (what == "bone_parent") {
 			r_ret = get_bone_parent(which);
+			return true;
 		} else if (what == "tail_direction") {
 			r_ret = get_tail_direction(which);
+			return true;
 		} else if (what == "bone_tail") {
 			r_ret = get_bone_tail(which);
+			return true;
 		} else if (what == "reference_pose") {
 			r_ret = get_reference_pose(which);
+			return true;
 		} else if (what == "handle_offset") {
 			r_ret = get_handle_offset(which);
+			return true;
 		} else if (what == "group") {
 			r_ret = get_group(which);
+			return true;
 		} else if (what == "require") {
 			r_ret = is_required(which);
-		} else {
-			return false;
+			return true;
 		}
 	}
-	return true;
+
+	return false;
 }
 
 void SkeletonProfile::_validate_property(PropertyInfo &p_property) const {
+	if (!Engine::get_singleton()->is_editor_hint()) {
+		return;
+	}
 	if (is_read_only) {
 		if (p_property.name == ("group_size") || p_property.name == ("bone_size") || p_property.name == ("root_bone") || p_property.name == ("scale_base_bone")) {
 			p_property.usage = PROPERTY_USAGE_NO_EDITOR;
@@ -132,17 +146,13 @@ void SkeletonProfile::_validate_property(PropertyInfo &p_property) const {
 	if (p_property.name == ("root_bone") || p_property.name == ("scale_base_bone")) {
 		String hint = "";
 		for (int i = 0; i < bones.size(); i++) {
-			hint += i == 0 ? String(bones[i].bone_name) : "," + String(bones[i].bone_name);
+			if (i > 0) {
+				hint += ",";
+			}
+			hint += String(bones[i].bone_name);
 		}
 		p_property.hint_string = hint;
 	}
-
-	PackedStringArray split = p_property.name.split("/");
-	if (split.size() == 3 && split[0] == "bones") {
-		if (split[2] == "bone_tail" && get_tail_direction(split[1].to_int()) != TAIL_DIRECTION_SPECIFIC_CHILD) {
-			p_property.usage = PROPERTY_USAGE_NONE;
-		}
-	}
 }
 
 void SkeletonProfile::_get_property_list(List<PropertyInfo> *p_list) const {
@@ -161,19 +171,17 @@ void SkeletonProfile::_get_property_list(List<PropertyInfo> *p_list) const {
 	}
 	for (int i = 0; i < bones.size(); i++) {
 		String path = "bones/" + itos(i) + "/";
+		int bone_tail_usage = (get_tail_direction(i) != TAIL_DIRECTION_SPECIFIC_CHILD) ? PROPERTY_USAGE_NONE : PROPERTY_USAGE_DEFAULT;
+
 		p_list->push_back(PropertyInfo(Variant::STRING_NAME, path + "bone_name"));
 		p_list->push_back(PropertyInfo(Variant::STRING_NAME, path + "bone_parent"));
 		p_list->push_back(PropertyInfo(Variant::INT, path + "tail_direction", PROPERTY_HINT_ENUM, "AverageChildren,SpecificChild,End"));
-		p_list->push_back(PropertyInfo(Variant::STRING_NAME, path + "bone_tail"));
+		p_list->push_back(PropertyInfo(Variant::STRING_NAME, path + "bone_tail", PROPERTY_HINT_NONE, "", bone_tail_usage));
 		p_list->push_back(PropertyInfo(Variant::TRANSFORM3D, path + "reference_pose"));
 		p_list->push_back(PropertyInfo(Variant::VECTOR2, path + "handle_offset"));
 		p_list->push_back(PropertyInfo(Variant::STRING_NAME, path + "group", PROPERTY_HINT_ENUM, group_names));
 		p_list->push_back(PropertyInfo(Variant::BOOL, path + "require"));
 	}
-
-	for (PropertyInfo &E : *p_list) {
-		_validate_property(E);
-	}
 }
 
 StringName SkeletonProfile::get_root_bone() {
@@ -266,6 +274,14 @@ int SkeletonProfile::find_bone(const StringName &p_bone_name) const {
 	return -1;
 }
 
+PackedStringArray SkeletonProfile::get_bone_names() {
+	PackedStringArray s;
+	for (const SkeletonProfileBone &bone : bones) {
+		s.push_back(bone.bone_name);
+	}
+	return s;
+}
+
 StringName SkeletonProfile::get_bone_name(int p_bone_idx) const {
 	ERR_FAIL_INDEX_V(p_bone_idx, bones.size(), StringName());
 	return bones[p_bone_idx].bone_name;
