@@ -28,8 +28,7 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef XR_SERVER_H
-#define XR_SERVER_H
+#pragma once
 
 #include "core/object/ref_counted.h"
 #include "core/os/thread_safe.h"
@@ -252,5 +251,3 @@ public:
 
 VARIANT_ENUM_CAST(XRServer::TrackerType);
 VARIANT_ENUM_CAST(XRServer::RotationMode);
-
-#endif // XR_SERVER_H
